/*
  Description: Use internal 2.048 V FVR for A/D conversion
  of LM34 temperature sensor
  MCU: PIC16F1827, 500 KHz internal clock, MCLR enabled
  Jul 2, 2011

*/

// Define LCD connections
sbit LCD_RS at RB2_bit;
sbit LCD_EN at RB3_bit;
sbit LCD_D4 at RB4_bit;
sbit LCD_D5 at RB5_bit;
sbit LCD_D6 at RB6_bit;
sbit LCD_D7 at RB7_bit;
sbit LCD_RS_Direction at TRISB2_bit;
sbit LCD_EN_Direction at TRISB3_bit;
sbit LCD_D4_Direction at TRISB4_bit;
sbit LCD_D5_Direction at TRISB5_bit;
sbit LCD_D6_Direction at TRISB6_bit;
sbit LCD_D7_Direction at TRISB7_bit;
// End LCD module connections

// Define Messages
 char message1[] = "Temp.";
 char *Temp = "000.0 F";
 unsigned ADC_Value, Temp_F;

void main() {

 ANSELA = 0b00000001;  // RA0 analog input
 TRISA  = 0b00100001;  // RA0, RA5 inputs
 ANSELB = 0b00000000;  // PORTB all digital outputs
 TRISB  = 0b00000000;  // PORTB all outputs
 
 // Configure FVR to 2.048 V
  FVRCON = 0b11000010 ;

  // Configure ADCON1
  ADCON1.ADNREF  = 0; // Vref- is connected to ground
  ADCON1.ADPREF0 = 1; // Vref+ is 4.096 V
  ADCON1.ADPREF1 = 1;
  ADCON1.ADCS0   = 0; // Use conversion clock, Fosc/2
  ADCON1.ADCS1   = 0; // Fosc = 500 KHz
  ADCON1.ADCS2   = 0;
  ADCON1.ADFM    = 1;  // result is right Justified

  // Configure ADCON0 for channel AN0
  ADCON0.CHS0 = 0;
  ADCON0.CHS1 = 0;
  ADCON0.CHS2 = 0;
  ADCON0.CHS3 = 0;
  ADCON0.CHS4 = 0;
  ADCON0.ADON = 1;  // enable A/D converter

  // Initialize LCD
  Lcd_Init();        // Initialize LCD
  Lcd_Cmd(_LCD_CLEAR);             // CLEAR display
  Lcd_Cmd(_LCD_CURSOR_OFF);        // Cursor off
  Lcd_Out(1,2,message1);            // Write message1 in 1st row

 do {
  
  ADCON0.F1 = 1;     // start conversion, GO/DONE = 1
  while (ADCON0.F1); // wait for conversion
  ADC_Value = (ADRESH << 8) + ADRESL;
  Temp_F = ADC_Value * 10/5;
  Temp[0] = Temp_F/1000 + 48;
  Temp[1] = (Temp_F/100)%10 + 48;
  Temp[2] = (Temp_F/10)%10 + 48;
  Temp[4] = Temp_F%10 + 48;
  if (Temp[0] == '0')                       // if Temp[0] = '0' then
      Temp[0] = ' ';                        // insert blank character to Temp[0]
  if (Temp[0] == ' ' && Temp[1] == '0')     // if Temp[0] is blank and Temp[1] =
      Temp[1] = ' ';                                       // '0' then
  Temp[5] = 223;
  LCD_Out(2,2, Temp);
  Delay_ms(1000);
 } while(1);


}